/***************************************************************************
                          kmatedit.cpp  -  description
                             -------------------
    begin                : Sam Mr 10 16:19:19 CET 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmatedit.h"
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qstring.h>
#include <qtabwidget.h>
#include <qcombobox.h>
#include <qcolor.h>
#include <qcheckbox.h>
#include <kcolorbutton.h>

Kmatedit::Kmatedit(QWidget *parent, const char *name) : MainDlg(parent, name)
{
	// make lighttab enviroment ready
	prepareLight();
}

Kmatedit::~Kmatedit()
{
	if(lightsourceattribs != 0) delete lightsourceattribs;
}

/** generates the text output */
void Kmatedit::generateText()
{
	// first we check if the name of the scene object was given by the user...
	if(SceneObjEntry->text().isEmpty())
	{
		QMessageBox::warning(this,"kMatEdit","You have to enter the name of the\n desired scene object!",
								QMessageBox::Ok,QMessageBox::NoButton);
		SceneObjEntry->setFocus();
		return;
	}
	else
	{
		// so, clear output widget first
		MainOutput->clear();
		
		// now lets determine which tab was selected
		if(TabObjectType->currentPage() == lights) MainOutput->insertLine(makeLight());
		if(TabObjectType->currentPage() == camera) MainOutput->insertLine(makeCamera());
		
	}
	
}
/** generates the source code for a lightflow light source */
QString Kmatedit::makeLight()
{
	QString sequence = SceneObjEntry->text() + ".newLight(";
	QString tripel;
	
	// now insert light source type
	sequence = sequence + "\"" + LightType->currentText() + "\",[";
	// now lets fill the option block
	
	// all lightsources have at least the color parameter...
	sequence = sequence + "\"color\",vector3(";
	// now we have to code the rgb info...
	QColor color = LightColorEntry->color();
	int intensity = LightIntensityEntry->value();
	sequence = sequence + tripel.sprintf("%f,%f,%f)",color.red()*intensity / 255.0f,
																										color.green()*intensity / 255.0f,
																										color.blue()*intensity / 255.0f);
	
	// all other parameters depends on the light source type
	lightattrib_t lattribs = lightsourceattribs[(lightsource_t) LightType->currentItem()];
	
	// is the voulumetric pattern parameter required?
	if(lattribs & cpattern)
	{
		sequence = sequence + ",\"color\",\"";
		sequence = sequence + LightAmbientPatternEntry->text();
		sequence = sequence + "\"";
	}
	
  // is the patchpos parameter required ?
	if(lattribs & patch_pos)
    {
      sequence = sequence + ",\"position\",vector3(";
      sequence = sequence + tripel.sprintf("%f,%f,%f)",LightPatchP1XEntry->text().toFloat(),
      																								 LightPatchP1YEntry->text().toFloat(),
      																								 LightPatchP1ZEntry->text().toFloat());
      sequence = sequence + ",vector3(";
			sequence = sequence + tripel.sprintf("%f,%f,%f)",LightPatchP2XEntry->text().toFloat(),
      																								 LightPatchP2YEntry->text().toFloat(),
      																								 LightPatchP2ZEntry->text().toFloat());
      sequence = sequence + ",vector3(";
			sequence = sequence + tripel.sprintf("%f,%f,%f)",LightPatchP3XEntry->text().toFloat(),
      																								 LightPatchP3YEntry->text().toFloat(),
      																								 LightPatchP3ZEntry->text().toFloat());
      sequence = sequence + ",vector3(";
			sequence = sequence + tripel.sprintf("%f,%f,%f)",LightPatchP4XEntry->text().toFloat(),
      																								 LightPatchP4YEntry->text().toFloat(),
      																								 LightPatchP4ZEntry->text().toFloat());
    }
  // is the patch type parameter required?
  if(lattribs & patch_sides)
  {
  	sequence = sequence + ",\"type\",";
  	sequence = sequence + ((LightPatchTwoSidesCheck->isChecked()) ? "\"two-sided\"" :"\"one-sided\"");
  }
	// is the position parameter required?
  if(lattribs & position)
    {
      sequence = sequence + ",\"position\",vector3(";
      sequence = sequence + tripel.sprintf("%f,%f,%f)",LightPosXEntry->text().toFloat(),
      																								 LightPosYEntry->text().toFloat(),
      																								 LightPosZEntry->text().toFloat());
    }
	// is the direction parameter required?
	if(lattribs & direction)
    {
      sequence = sequence + ",\"direction\",vector3(";
      sequence = sequence + tripel.sprintf("%f,%f,%f)",LightDirXEntry->text().toFloat(),
      																								 LightDirYEntry->text().toFloat(),
      																								 LightDirZEntry->text().toFloat());
    }
	// spotlight?
	if(lattribs & spotattribs)
		{
			sequence = sequence + ",\"angle\",";
			sequence = sequence + tripel.sprintf("%f",toRadians(LightAngleEntry->value()));
			
			sequence = sequence + tripel.sprintf(",%f",toRadians(LightPenumbraAngle->value()));
	
			sequence = sequence + ",\"exponent\",";
			sequence = sequence + tripel.sprintf("%f",LightPenumbraExponent->text().toFloat());
		}
	
	// is the decay value required and not equal to default = 2?
	if((lattribs & decay) && (LightDecayEntry->value() != 2))
	{
		sequence = sequence + ",\"decay\",";
		sequence = sequence + tripel.sprintf("%f",(float) LightDecayEntry->value());
	}
	
	// is the shadow flag required?
	if(lattribs & shadows)
	{
		sequence = sequence + ",\"shadows\",";
		sequence = sequence + ((LightShadowCheck->isChecked() == true) ? "1" : "0");
	}
	
	// soft light shadow parameters required?
	if(lattribs & softshadowattribs)
	{
		sequence = sequence + ",\"zbuffer\",";
		sequence = sequence + tripel.sprintf("%i",LightSoftZBuffer->value());
	
		sequence = sequence + ",\"radius\",";
		sequence = sequence + tripel.sprintf("%f",LightSoftRadius->value() / 100.0f);

		sequence = sequence + ",\"samples\",";
		sequence = sequence + tripel.sprintf("%i",LightSoftSamples->value());
	
		sequence = sequence + ",\"bias\",";
		sequence = sequence + tripel.sprintf("%f",LightSoftBias->text().toFloat());
	
	// finally, the file parameter if allowed
		if(LightSoftFileAttribCheck->isChecked())
		{
			sequence = sequence + ",\"file\",\"";
			sequence = sequence + LightSoftFileEntry->text();
			sequence = sequence + "\",\"";
			sequence = sequence + LightSoftFileFlagEntry->currentText();
			sequence = sequence + "\"";
		}
	}
	
	// terminating sequence
	sequence = sequence + "])";
	return sequence;
}



char *Kmatedit::lightsourceexp[] = {"ambient / background light",
																		 "sharp spot light",
																		 "for sunlight e.g.",
																		 "to simulate windows e.g.",
													           "to simulate neon fixtures e.g.",
													           "sharp point light",
													           "soft point lights",
													           "soft spot-lights",
													           "for soft sunlight e.g."};

static char *Kmatedit::imagesavers[] = {"tga-saver","saver"};
													
/** disables all widgets in the light sheed 
	*	which are not required for the selected
	* light source type
	*/
void Kmatedit::lightDisableCheck(int index)
{
	lightattrib_t lattribs = lightsourceattribs[index];
	
	// now go to the map of widgets and enable / disable them
	typedef map<QWidget *,lightattrib_t>::const_iterator CI;
	
	for(CI p=widgetallow.begin(); p != widgetallow.end(); ++p)
	{
		// enable widget if a attribute which required it is set, otherwise disable it
		p->first->setEnabled(((p->second & lattribs) != 0));
	}
	
	if(LightSoftFileAttribCheck->isEnabled())
	{
		bool temp = LightSoftFileAttribCheck->isChecked();
		LightSoftFileEntry->setEnabled(temp);
		LightSoftFileFlagEntry->setEnabled(temp);
	}
	
	if(LightAmbientPatternCheck->isEnabled())
	{
		LightAmbientPatternEntry->setEnabled(LightAmbientPatternCheck->isChecked());
	}
}

/** initialise light tab enviroment */
void Kmatedit::prepareLight()
{
		// make an array for the 9 available lightsources
	lightsourceattribs = new lightattrib_t[9];
	
	// add all used attribues to the lightsource entries
	lightsourceattribs[ambient] = cpattern;
	lightsourceattribs[conic] = position | direction | shadows | spotattribs | decay;
	lightsourceattribs[directional] = direction | shadows;
	lightsourceattribs[directional_patch] = patch_pos | direction | shadows;
	lightsourceattribs[patch] = patch_pos | patch_sides | decay | shadows;
	lightsourceattribs[point] = position | decay | shadows;
	lightsourceattribs[soft] = position | decay | softshadowattribs;
	lightsourceattribs[soft_conic] = position | direction | spotattribs | decay | softshadowattribs;
	lightsourceattribs[soft_directional] = direction | softshadowattribs;
	
	// create a map of widgets which are not always used to the attributes which requires them
	widgetallow[LightShadowCheck] = shadows;
	widgetallow[LightPosXEntry] = position;
	widgetallow[LightPosYEntry] = position;
	widgetallow[LightPosZEntry] = position;
	widgetallow[LightDirXEntry] = direction;
	widgetallow[LightDirYEntry] = direction;
	widgetallow[LightDirZEntry] = direction;
	widgetallow[LightPatchP1XEntry] = patch_pos;
	widgetallow[LightPatchP1YEntry] = patch_pos;
	widgetallow[LightPatchP1ZEntry] = patch_pos;
	widgetallow[LightPatchP2XEntry] = patch_pos;
	widgetallow[LightPatchP2YEntry] = patch_pos;
	widgetallow[LightPatchP2ZEntry] = patch_pos;
	widgetallow[LightPatchP3XEntry] = patch_pos;
	widgetallow[LightPatchP3YEntry] = patch_pos;
	widgetallow[LightPatchP3ZEntry] = patch_pos;
	widgetallow[LightPatchP4XEntry] = patch_pos;
	widgetallow[LightPatchP4YEntry] = patch_pos;
	widgetallow[LightPatchP4ZEntry] = patch_pos;
	widgetallow[LightPatchTwoSidesCheck] = patch_sides;
	widgetallow[LightAngleEntry] = spotattribs;
	widgetallow[LightPenumbraAngle] = spotattribs;
	widgetallow[LightPenumbraExponent] = spotattribs;
	widgetallow[LightDecayEntry] = decay;
	widgetallow[LightSoftBias] = softshadowattribs;
	widgetallow[LightSoftFileAttribCheck] = softshadowattribs;
	widgetallow[LightSoftFileEntry] = softshadowattribs;
	widgetallow[LightSoftFileFlagEntry] = softshadowattribs;
	widgetallow[LightSoftZBuffer] = softshadowattribs;
	widgetallow[LightSoftSamples] = softshadowattribs;
	widgetallow[LightSoftRadius] = softshadowattribs;
	widgetallow[LightAmbientPatternCheck] = cpattern;
	widgetallow[LightAmbientPatternEntry] = cpattern;
	
	// initiate check
	lightDisableCheck(0);	
}
/** generates the source code for a lightflow camera & saver enviroment */
QString Kmatedit::makeCamera()
{
	QString sequence;
	QString numbers;
	
	// filter imager required?
	if(CameraFilterEnableCheck->isChecked())
	{
		sfxfilter_t filter = (sfxfilter_t) CameraFilterSelTab->currentPageIndex();
		if(CameraAddFilmCheck->isChecked() && (filter != film))
		{
			// user wishes additional film effect filter
			sequence = "prefilter = " + makeCameraFilter(filter) + "\n";
			// make film filter as sfxfilter
			sequence = sequence + "sfxfilter  = " + SceneObjEntry->text() + ".newImager(\"film\",[\"imager\",prefilter,\"grain\",";
			sequence = sequence + numbers.sprintf("%f])\n\n",CameraFilmGrainEntry->text().toFloat());
		}
		else
		{
			// no additional film filter or film filter selected twice (filter + addition)
		 	sequence = "sfxfilter = " + makeCameraFilter(filter) + "\n\n";
		}
	}
	
	// o.k. make the saver
	sequence = sequence + "saver = " + SceneObjEntry->text();
	sequence = sequence +  + ".newImager(\"";
	sequence = sequence + imagesavers[CameraTypeCombo->currentItem()];
	sequence = sequence + "\",[\"file\",\"";
	sequence = sequence + CameraFileEntry->text();
	sequence = sequence + CameraTypeCombo->currentText();
	sequence = sequence + "\"";
	
	if(CameraFilterEnableCheck->isChecked())
	{
		sequence = sequence + ",\"imager\",sfxfilter";
	}
	
	if(CameraAlphaCheck->isEnabled())
	{
		sequence = sequence + ",\"alpha\",";
		sequence = sequence + ((CameraAlphaCheck->isChecked()) ? "1" : "0");
	}
	sequence = sequence + "])\n\n";
	
	// now comes the camera
	sequence = sequence + SceneObjEntry->text();
	sequence = sequence + ".imagerBegin(saver)\n";
	// generate camera
	sequence = sequence + "camera = " + SceneObjEntry->text();
	sequence = sequence + ".newCamera(\"pinhole\",[\"eye\",";
	sequence = sequence + "vector3(" + CameraPosX->text() + "," + CameraPosY->text() + "," + CameraPosZ->text() + ")";
	sequence = sequence + ",\"aim\",vector3(" + CameraAimtoX->text() + "," + CameraAimtoY->text() + "," + CameraAimtoZ->text() + ")";
	// include aa parameters if desired
	if(CameraAASamplesCheck->isChecked())
	{
		sequence = sequence + ",\"aa-samples\"," + CameraAAareaSpin_2->text() + "," + CameraAAsupersamplingSpin->text();
	}
	if(CameraAAThresholdCheck->isChecked())
	{
	 	sequence = sequence + ",\"aa-threshold\"," + numbers.sprintf("%f",CameraAAThresholdEntry->text().toFloat());
	}
	if(CameraAAJitterCheck->isChecked())
	{
		sequence = sequence + ",\"aa-jitter\"," + numbers.sprintf("%f",CameraAAJitterEntry->text().toFloat());
	}
	// include extended view parameters if desired
	if(CameraViewUpCheck->isChecked())
	{
		sequence = sequence + ",\"up\",vector3(" + CameraViewUpX->text() + "," + CameraViewUpY->text() + "," + CameraViewUpZ->text() + ")";
	}
	if(CameraViewFovCheck->isChecked())
	{
		sequence = sequence + ",\"fov\"," + numbers.sprintf("%f",CameraViewVertEntry->text().toFloat());
		if(CameraViewFovHorizEntry->edited())
		{
			sequence = sequence + ",\"fov\",\"horizontal\"," + numbers.sprintf("%f",CameraViewFovHorizEntry->text().toFloat());
		}
	}
	if(CameraViewApertureCheck->isChecked())
	{
		sequence = sequence + ",\"aperture\"," + numbers.sprintf("%f",CameraViewApertureEntry->text().toFloat());
	}
	if(CameraViewAspectCheck->isChecked())
	{
		sequence = sequence + ",\"aspect\"," + numbers.sprintf("%f",CameraViewAspectEntry->text().toFloat());
	}
	sequence = sequence + "])\n";
	sequence = sequence + SceneObjEntry->text();
	sequence = sequence + ".imagerEnd(saver)\n\n";
	
	// enable radiosity?
	if(CameraRadiosityCheck->isChecked())
	{
		sequence = sequence + SceneObjEntry->text();
		sequence = sequence + ".radiosity()\n\n";
	}
	
	// give the render command
	sequence = sequence + SceneObjEntry->text();
	sequence = sequence + ".render(camera,";
	sequence = sequence + CameraResXEntry->text() + "," + CameraResYEntry->text();
	sequence = sequence + ")\n";
	return(sequence);
}

/**
	* disables / enables widgets dependent on the selected
	* widget type
	*/
	
void Kmatedit::cameraDisableCheck(int selection)
{
	// enable alpha channel option only for tga saver
	CameraAlphaCheck->setEnabled(selection == 0);
}

/**
	* generates the code for an imager filter
	* @param filter index of activated filter tab page
	*/
QString Kmatedit::makeCameraFilter(sfxfilter_t filter)
{
	QString numbers;
	QString sequence = SceneObjEntry->text() + ".newImager(\"";
	sequence = sequence + CameraFilterSelTab->tabLabel(CameraFilterSelTab->currentPage());
	sequence = sequence + "\",[";
	switch(filter)
	{
		case dof :
			sequence = sequence + "\"depth\"," + ((CameraDofAutofocusCheck->isChecked()) ? QString("-1.0") : numbers.sprintf("%f",CameraDofDepthEntry->text().toFloat()));
			sequence = sequence + ",\"radius\"," + numbers.sprintf("%f",CameraDofRadiusEntry->text().toFloat());
			sequence = sequence + ",\"mask\"," + CameraDofMaskSpin->text();
			break;
		case film :
			sequence = sequence + "\"grain\"," + numbers.sprintf("%f",CameraFilmGrainEntry->text().toFloat());
			break;
		case glitter :
			sequence = sequence + "\"radius\"," + numbers.sprintf("%f",CameraGlitterRadiusEntry->text().toFloat());
			sequence = sequence + ",\"intensity\"," + numbers.sprintf("%f",CameraGlitterIntensityEntry->text().toFloat());
			sequence = sequence + ",\"threshold\"," + numbers.sprintf("%f",CameraGlitterThresholdEntry->text().toFloat());
			break;
		case halo :
			sequence = sequence + "\"kr\"," + numbers.sprintf("%f",CameraHaloKrEntry->text().toFloat());
			sequence = sequence + ",\"km\"," + numbers.sprintf("%f",CameraHaloKmEntry->text().toFloat());
			sequence = sequence + ",\"shininess\"," + numbers.sprintf("%f",CameraHaloShEntry->text().toFloat());
	}	
	sequence = sequence + "])";	
	return(sequence);
}
